--[[==========================================
	Recount
	==========================================
	A damage meter addon.
	Based on Fayn's work	
	updated and maintained by McGuffin since 0.4.0
  ]]--



Recount = {}
Recount.name = "Recount"
Recount.command = "/recount"
Recount.versionString = "v0.5.0"
Recount.versionSettings = 9;
Recount.versionBuild = 24;

Recount.time = 0;
Recount.refreshThreshold = 250
Recount.inCombat = false
Recount.onExitCombat = false
Recount.lastUpdateUI = 0;
Recount.isDebugMode = false;
Recount.preMinMode = RC_MODE_COMBAT_LOG;
Recount.isResizing = false;
Recount.preCombatMode = RC_MODE_DAMAGE_DONE;

Recount.totalMode =false;
Recount.topBarId = 1;
Recount.maxComputedBars = 32;
Recount.UIinMenu = false;

local EVENT_TYPE_IGNORE = 0;
local EVENT_TYPE_DAMAGE = 1;
local EVENT_TYPE_HEAL = 2;
local EVENT_TYPE_DAMAGE_DEFLECTED = 3;

local RC_MODE_DAMAGE_DONE = 0;
local RC_MODE_DAMAGE_DONE_TOTAL = 1;
local RC_MODE_DAMAGE_TAKEN = 2;
local RC_MODE_DAMAGE_TAKEN_TOTAL = 3;
local RC_MODE_HEALING_DONE = 4;
local RC_MODE_HEALING_DONE_TOTAL = 5;
local RC_MODE_HEALING_TAKEN = 6;
local RC_MODE_HEALING_TAKEN_TOTAL = 7;
local RC_MODE_COMBAT_LOG = 8;
local RC_MODE_EOL = 9;
local RC_MODE_MINIMIZED = 10;


local modeStrings = {
	[RC_MODE_DAMAGE_DONE] = "Damage Done last Fight",
	[RC_MODE_DAMAGE_DONE_TOTAL] = "Total Damage Done",
	[RC_MODE_DAMAGE_TAKEN] = "Damage Taken last Fight",
	[RC_MODE_DAMAGE_TAKEN_TOTAL] = "Total Damage Taken",
	[RC_MODE_HEALING_DONE] = "Healing Done last Fight",
	[RC_MODE_HEALING_DONE_TOTAL] = "Total Healing Done",
	[RC_MODE_HEALING_TAKEN] = "Healing Taken last Fight",
	[RC_MODE_HEALING_TAKEN_TOTAL] = "Total Healing Taken",	
	[RC_MODE_COMBAT_LOG] = "Combat Log",	
	[RC_MODE_MINIMIZED] = "n/a",	
}


local UI_STATUSBAR_MAX = 32;
local UI_STATUSBAR_HEIGHT = 25;
local UI_TITLEBAR_HEIGHT = 25;

Recount.settingsDefaults = {
	numberFormat = 1;
	currentMode = RC_MODE_DAMAGE_DONE_TOTAL;
	numStoredSessions = 10;
	autoSwitchMode = true;	
	dotTag = "^";
	critTag =  "*";	
	minimizeDuringCombat = false;
	showSkillDetails = true;
	hideOutOfCombat = false;
	hideOutOfCombatDelayInSeconds = 15;
	suspendDataCollectionWhileHidden = false;
	onlyUseKey = false;
	
	--totalStatusLabel = "Total #td eDps: #edps (#tdcrit%) | #th eHps: #ehps (#thc) [#tdur]"
	--lastStatusLabel = "Last Dps: #dps (#ldcrit) | Hps: #hps (#lhcrit) [#ldur]"

	countDeflectedDmg = false;
	
	progressBarHeight = 25;
	progressBarFont = "ZoFontGameSmall"  ;--"ZoFontBoss";
	progressBarColor = {
		r = 0.8,
		g = 0.8,
		b = 0.5,
		a = 0.85
	};
	
	wndMain = {
		x = 10;
		y = 10;
		width = 250;
		height = 160;
		isVisible = true;
	}
}

-- contains elements similar to playerUnit, unit 1 is always the player itself
Recount.trackedUnits = {};

--[[==========================================
	playerUnit
	Prototype for the unit structure
	==========================================]]--
playerUnit = {
	displayName = string.gsub( GetUnitName( 'player' ) , "%-", "%%%-");

	sessionList = {};
	sessionCurrent = 1;
	sessionNum = 0;
	session = nil;

	total = {
		damageDone = 0;
		damageTaken = 0;
		healingDone = 0;
		healingTaken = 0;
		
		healingCount = 0;
		critHealingCount = 0;
		damageCount = 0;
		critDamageCount = 0;
		
		dps = 0;
		hps = 0;

		timeStart = 0;
		timeSpan = 0;

		damagePerAbility = {};
		healPerAbility = {};
	};
}


--[[==========================================
	Initialize
	==========================================]]--
function Recount.Initialize( eventCode, addOnName )
	-- Only set up for Recount
	if ( addOnName ~= Recount.name ) then
		return
	end
	
	Recount.settings = ZO_SavedVars:New( "RecountSettings" , Recount.versionSettings, nil, Recount.settingsDefaults, nil );
		
	EVENT_MANAGER:RegisterForEvent( Recount.name, EVENT_COMBAT_EVENT, Recount.CombatEvent );	
	EVENT_MANAGER:RegisterForEvent( Recount.name, EVENT_PLAYER_COMBAT_STATE, Recount.CombatStateEvent );	
	EVENT_MANAGER:RegisterForEvent( Recount.name, EVENT_PLAYER_DEAD, Recount.playerExitCombat );	
	
	
	
	SLASH_COMMANDS[Recount.command] = Recount.SlashCommands;
	
	
	-- autoHide manager, use cursor events--------->
	--EVENT_MANAGER:RegisterForEvent( Recount.name,EVENT_ACTION_LAYER_PUSHED, Recount.autoToggleHide);
	--EVENT_MANAGER:RegisterForEvent( Recount.name,EVENT_ACTION_LAYER_POPPED, Recount.autoToggleShow);
	
	-- or... lets link to the hud instead, so it will switch nicely
	local  	w = WINDOW_MANAGER:CreateTopLevelWindow()
			w:SetDimensions(1,1)
			w:SetAnchor(CENTER, GuiRoot, CENTER, 0, 0)
			w:SetHidden(true)
    local ssf = ZO_SimpleSceneFragment:New(w)
	local h = SCENE_MANAGER:GetScene("hud")
    local ui = SCENE_MANAGER:GetScene("hudui")
    h:AddFragment(ssf)
    ui:AddFragment(ssf)
    ssf:RegisterCallback("StateChange", Recount.autoToggle)
	--

	Recount.InitUI();

	-- never start in minimized mode
	if ( Recount.settings.currentMode == RC_MODE_MINIMIZED ) then
		Recount.settings.currentMode = RC_MODE_COMBAT_LOG;
	end
	Recount.SetMode( Recount.settings.currentMode, true );
		
	-- first tracked unit it always the player
	Recount.trackedUnits[1] = playerUnit;	

	-- set up the addon settings menu
	Recount.SettingsMenu.Initialize();
		
	Recount.CombatLogMsg( string.format( "Recount (%s) loaded.", Recount.versionString ) );
	Recount.CombatLogMsg( "Press the 'M' button in the top right corner to switch between modes." );
	Recount.CombatLogMsg( "Press the 'R' button in the top right corner to reset Recount." );
	
	--if Recount.isDebugMode then
	--	Recount.TestProgressBars()
	--end
end

-- Hook initialization onto the ADD_ON_LOADED event
EVENT_MANAGER:RegisterForEvent( Recount.name, EVENT_ADD_ON_LOADED, Recount.Initialize )


--[[==========================================
	FormatNumber

	Shortens large integers into ..k or ..m text
	==========================================]]--
function Recount.FormatNumber( number )
	if number then
		if Recount.settings.numberFormat == 1 then
			if number > 999999 then
				return 	("%02.2fM"):format(number / 1000000);
			elseif number > 999 then
				return 	("%02.1fK"):format(number / 1000);
			else
				return "" .. math.floor(number);
			end
		else
			return "" .. math.floor(number);
		end
	else
		return ""
	end
end

function convertTimeStamp(ms)
	local s = (ms / 1000) % 60;
	local mn = (ms / (1000 * 60)) % 60;
	local ms = (ms % 1000)
	return string.format(" +%02d:%02d:%03d ", mn, s, ms);
end

function convertTimeStampS(ms)
	local s = (ms / 1000) % 60;
	local mn = (ms / (1000 * 60));
	return string.format("%d:%02d", mn, s);
end

function Recount.InitUI()
	-- RecountUI:SetResizeHandleSize( MOUSE_CURSOR_RESIZE_NS );

	if ( Recount.settings.wndMain.width < 10 ) then
		Recount.settings.wndMain.width = Recount.settingsDefaults.wndMain.width;
	end

	if ( Recount.settings.wndMain.x < 0 ) then
		Recount.settings.wndMain.x = 0;
	end

	if ( Recount.settings.wndMain.x > GuiRoot:GetWidth() ) then
		Recount.settings.wndMain.x = GuiRoot:GetWidth() - Recount.settings.wndMain.width;
	end

	RecountUI:SetAnchor( TOPLEFT, GuiRoot, TOPLEFT, Recount.settings.wndMain.x, Recount.settings.wndMain.y );	
	RecountUI:SetWidth( Recount.settings.wndMain.width );
	RecountUI:SetHeight( Recount.settings.wndMain.height );
	
-- Recount.topBarId
	for i = 1, UI_STATUSBAR_MAX do
		local statusBar = CreateControlFromVirtual( "Statusbar", RecountUI, "StatusbarTemplate", i );
		_G["Statusbar"..i]:SetAnchor( TOPLEFT, RecountUI, TOPLEFT, 0, i * UI_STATUSBAR_HEIGHT );
		_G["Statusbar"..i]:SetMouseEnabled(true)
		_G["Statusbar"..i]:SetHandler("OnMouseWheel" , Recount.scrollBars)
		--_G["Statusbar"..i]:SetHandler("OnMouseEnter",  Recount.barsMouseEnter)	
	end
	
	RecountUIDebug:SetHidden( true );
	
	if Recount.settings.hideOutOfCombat then
		Recount.settings.wndMain.isVisible= false
	end
	
	if ( Recount.settings.wndMain.isVisible == false ) then
		Recount.Hide();
	end
	
	Recount.FitUIElements();
end

function Recount.scrollBars(self, delta) 
	Recount.topBarId = Recount.topBarId - delta	
	local clientHeight = RecountUI:GetHeight() - UI_TITLEBAR_HEIGHT - UI_STATUSBAR_HEIGHT;
	local numBars = clientHeight / Recount.settings.progressBarHeight;
	if Recount.topBarId > Recount.maxComputedBars - numBars then
		Recount.topBarId = Recount.maxComputedBars - numBars
	end
	if Recount.topBarId < 1 then
		Recount.topBarId = 1
	end
	Recount.FitStatusBars()
end

function Recount.MinimizeWindow()
	if ( Recount.settings.currentMode == RC_MODE_MINIMIZED ) then
		return;
	end
	Recount.preMinMode = Recount.settings.currentMode;
	Recount.SetMode( RC_MODE_MINIMIZED, true );
end

function Recount.RestoreWindow(doUpdate)
	Recount.SetMode( Recount.preMinMode, doUpdate );
end

function Recount.OnResizeStart( self )
	Recount.isResizing = true;
end

function Recount.OnMoveStop( self )
	Recount.settings.wndMain.x = self:GetLeft();
	Recount.settings.wndMain.y = self:GetTop();
	Recount.settings.wndMain.width = self:GetWidth();
	Recount.settings.wndMain.height = self:GetHeight();
end

function Recount.OnResizeStop( self )
	Recount.OnMoveStop( self );
	Recount.FitUIElements();
	Recount.isResizing = false;	
end

function Recount.GetMinWidth()
	-- buttons + margin
	return 100 + 25 + 25 + 25;
end

function Recount.GetMinHeight()
	-- titlebar + statusbar + 1 progressbar
	return UI_STATUSBAR_HEIGHT + UI_TITLEBAR_HEIGHT + Recount.settings.progressBarHeight;
end

function Recount.FitStatusBars()
	-- calculate how many bars are visible
	local clientHeight = RecountUI:GetHeight() - UI_TITLEBAR_HEIGHT - UI_STATUSBAR_HEIGHT;
	local numBars = clientHeight / Recount.settings.progressBarHeight;
	local offset = 1
	for i = 1, UI_STATUSBAR_MAX do	
		_G["Statusbar"..i]:SetHidden( true );
		if (i >= Recount.topBarId) and (offset <= numBars) then	
			_G["Statusbar"..i]:SetAnchor( TOPLEFT, RecountUI, TOPLEFT, 2, ( offset-1) * Recount.settings.progressBarHeight + UI_TITLEBAR_HEIGHT );
			_G["Statusbar"..i]:SetWidth( Recount.settings.wndMain.width - 4);
			_G["Statusbar"..i]:SetHeight( Recount.settings.progressBarHeight );
			_G["Statusbar"..i]:SetColor( Recount.settings.progressBarColor.r, Recount.settings.progressBarColor.g, Recount.settings.progressBarColor.b, Recount.settings.progressBarColor.a );
			_G["Statusbar"..i.."_lblLeft"]:SetFont( Recount.settings.progressBarFont );
			_G["Statusbar"..i]:SetHidden( false );
			offset = offset + 1
		end		
	end
end

function Recount.FitUIElements()
	local w = RecountUI:GetWidth();
	local h = RecountUI:GetHeight();

	if ( w < Recount.GetMinWidth() ) then
		w = Recount.GetMinWidth();
		RecountUI:SetWidth( w );		
	end
	if ( h < Recount.GetMinHeight() ) then
		h = Recount.GetMinHeight();
		RecountUI:SetHeight( h );		
	end
	
	RecountUI_Title:SetWidth( w );
	RecountUI_lblStatus:SetWidth( w );

	RecountUI_lblCombatLogBG:SetWidth( w );
	RecountUI_lblCombatLogBG:SetHeight( h - UI_STATUSBAR_HEIGHT );

	RecountUI_lblCombatLog:SetWidth( w );
	RecountUI_lblCombatLog:SetHeight( h - UI_STATUSBAR_HEIGHT - UI_TITLEBAR_HEIGHT );

	Recount.FitStatusBars();
end


--[[==========================================
	UpdateUI
	==========================================]]--
function secureDivide(value1, value2)
	if value2 == 0 then
		return 0 
	else
		return (value1 / value2);
	end;
end

function getCritStat(data)
	return  secureDivide(data.critHealingCount, data.healingCount)*100, 
			secureDivide(data.critDamageCount, data.damageCount)*100;		
end	
	
function Recount.UpdateUI(ttimer)	
	if (ttimer == 0) or  (Recount.lastUpdateUI <= Recount.time) then
		Recount.lastUpdateUI = Recount.time + ttimer;
		Recount.doUpdateUI();
	end	
end

function getMinTimeSpan(t)
	if t < 1000 then
		return 1
	else 
		return t / 1000.0
	end
end
	
function Recount.doUpdateUI()
	if (Recount.totalMode) or (playerUnit.session == nil)   then
		local t = getMinTimeSpan(playerUnit.total.timeSpan)
		playerUnit.total.dps = playerUnit.total.damageDone / t;
		playerUnit.total.hps = playerUnit.total.healingDone / t;
		local critH, critD = getCritStat(playerUnit.total)
		RecountUI_lblStatus:SetText( string.format( "Total eDps: |ccccc55 %s|r (|cff5555 %.1f%% |r)  /   eHps: |c55cc55 %s|r (|cff5555 %.1f%% |r) - [%s]", Recount.FormatNumber( playerUnit.total.dps ) , critD, Recount.FormatNumber( playerUnit.total.hps ),critH , convertTimeStampS(playerUnit.total.timeSpan)) );
	else
		local t = getMinTimeSpan(playerUnit.session.timeSpan)
		playerUnit.session.dps = playerUnit.session.damageDone / t;
		playerUnit.session.hps = playerUnit.session.healingDone / t;
		local critH, critD = getCritStat(playerUnit.session)
		RecountUI_lblStatus:SetText( string.format( "Last DPS: |ccccc55 %s|r (|cff5555 %.1f%% |r)  /   HPS: |c55cc55 %s|r (|cff5555 %.1f%% |r) - [%s]", Recount.FormatNumber( playerUnit.session.dps ), critD,  Recount.FormatNumber( playerUnit.session.hps ), critH , convertTimeStampS(playerUnit.session.timeSpan) ) );
	end
	Recount.UpdateModeUI();
end

function Recount.OnUpdate( self )

	if ( Recount.isResizing ) then
		Recount.FitUIElements()		
	end

end

--[[==========================================

	==========================================]]--
function Recount.CombatLogMsg( msg )
	local tStamp = " "
	if playerUnit.session then	
		tStamp = convertTimeStamp(playerUnit.session.timeSpan)		
	end
	RecountUI_lblCombatLog:AddMessage(string.format("|c777777 %s |r  %s",tStamp,msg));
end
--[[==========================================

	==========================================]]--
	
function Recount.playerEnterCombat()
	EVENT_MANAGER:UnregisterForUpdate("hideOOC")
	if Recount.onExitCombat == true then
		Recount.onExitCombat = false
		return;
	end
	Recount.inCombat = true
	--new session
	Recount.StartNewSession();
	--compute total active time:
	if playerUnit.total.timeStart == 0 then
		playerUnit.total.timeStart = Recount.time
	else
		playerUnit.total.timeStart = Recount.time - playerUnit.total.timeSpan
	end
	--feedback
	Recount.CombatLogMsg( "+++ Combat +++" );
	if (Recount.settings.onlyUseKey == false)  then
		Recount.Show();
	end
	if ( Recount.settings.autoSwitchMode == true ) then
		Recount.preCombatMode = Recount.settings.currentMode;
		Recount.SetMode( RC_MODE_COMBAT_LOG, false );
	end
	if ( Recount.settings.minimizeDuringCombat == true ) then
		Recount.MinimizeWindow();
	end
	Recount.UpdateUI(0);
end 

function Recount.doPlayerExitCombat()		
	EVENT_MANAGER:UnregisterForUpdate("hideOOC")
	if Recount.onExitCombat == false then
		return;
	end
	Recount.time = GetGameTimeMilliseconds();		
	if (Recount.settings.hideOutOfCombat ==  true) and (Recount.settings.onlyUseKey == false)  then
		EVENT_MANAGER:RegisterForUpdate("hideOOC", Recount.settings.hideOutOfCombatDelayInSeconds * 1000, Recount.Hide)
	end
	if ( Recount.settings.minimizeDuringCombat == true ) then
		Recount.RestoreWindow(false);
	end
	if ( Recount.settings.autoSwitchMode == true ) then
		Recount.SetMode( Recount.preCombatMode, false );
	end		

	if ( playerUnit.session ~= nil ) then
		Recount.CombatLogMsg( string.format( "--- Combat [%ds] ---", playerUnit.session.timeSpan / 1000 ) ); 
		Recount.CombatLogMsg( string.format( "--- out: |ccccc55 %s|r / |c55cc55 %s|r; in: |c883333 %s|r dmg",
							  Recount.FormatNumber( playerUnit.session.damageDone ), Recount.FormatNumber( playerUnit.session.healingDone ), Recount.FormatNumber( playerUnit.session.damageTaken ) ) );
	else
		Recount.CombatLogMsg( "--- Combat [%ds] ---" ); 
	end	
	Recount.inCombat = false
	Recount.onExitCombat = false	
	Recount.UpdateUI(0);
end	

function Recount.playerExitCombat()
	if (Recount.inCombat == true) and (Recount.onExitCombat == false) then	
		Recount.onExitCombat = true;
		zo_callLater(Recount.doPlayerExitCombat, 1000)
	end
end
		
function Recount.CombatStateEvent( eventCode, inCombat )
	Recount.time = GetGameTimeMilliseconds();
	if ( inCombat == true ) then
		Recount.playerEnterCombat()
	else	
		Recount.playerExitCombat()
	end
end


--[[==========================================
	CheckIgnoreCombatEvent

	returns EVENT_TYPE_*
	==========================================]]--
function Recount.CheckIgnoreCombatEvent( result, isError, hitValue, powerType, damageType )

	-- ignore non-successful casts
	if ( isError ) then
		return EVENT_TYPE_IGNORE, 0, 0, 0, 0;
	end

	if ( hitValue == 0 ) then
		return EVENT_TYPE_IGNORE, 0, 0, 0, 0;
	end
	
	-- ignore certain powers
	if ( powerType == POWERTYPE_INVALID or powerType == POWERTYPE_MOUNT_STAMINA ) then
		return EVENT_TYPE_IGNORE, 0, 0, 0, 0;
	end
	
	if ( damageType == DAMAGE_TYPE_NONE ) then
		return EVENT_TYPE_IGNORE, 0, 0, 0, 0;
	end

	if ( result == ACTION_RESULT_HEAL ) then
		return EVENT_TYPE_HEAL, 1, 0, 0, 0;
	end 
	
	if ( result == ACTION_RESULT_CRITICAL_HEAL ) then
		return EVENT_TYPE_HEAL, 1, 1, 0, 0;
	end 
	
	if ( result == ACTION_RESULT_HOT_TICK ) then
		return EVENT_TYPE_HEAL, 0, 0, 1, 0;
	end 
	
	if ( result == ACTION_RESULT_HOT_TICK_CRITICAL ) then
		return EVENT_TYPE_HEAL, 0, 0, 1, 1;
	end 

	if ( result == ACTION_RESULT_BLOCKED or
		 result == ACTION_RESULT_DAMAGE_SHIELDED or
		 result == ACTION_RESULT_PARRIED or
		 result == ACTION_RESULT_REFLECTED or
		 result == ACTION_RESULT_IMMUNE ) then
		if ( Recount.settings.countDeflectedDmg ) then
			return EVENT_TYPE_DAMAGE, 0, 0, 0, 0;
		end
		return EVENT_TYPE_DAMAGE_DEFLECTED, 0, 0, 0, 0;
	end 

	if ( result == ACTION_RESULT_ABSORBED or
		 result == ACTION_RESULT_BLOCKED_DAMAGE or
		 result == ACTION_RESULT_FALL_DAMAGE or
		 result == ACTION_RESULT_PARTIAL_RESIST or
		 result == ACTION_RESULT_PRECISE_DAMAGE or
		 result == ACTION_RESULT_WRECKING_DAMAGE ) then
		return EVENT_TYPE_DAMAGE, 0, 0, 0, 0;
	end 
	
	if ( result == ACTION_RESULT_DAMAGE) then
		return EVENT_TYPE_DAMAGE, 1, 0, 0, 0;
	end 
	
	if ( result == ACTION_RESULT_CRITICAL_DAMAGE) then
		return EVENT_TYPE_DAMAGE, 1, 1, 0, 0;
	end 
	
	--dot
	if ( result == ACTION_RESULT_DOT_TICK) then
		return EVENT_TYPE_DAMAGE, 0, 0, 1, 0;
	end 
	
	if ( result == ACTION_RESULT_DOT_TICK_CRITICAL) then
		return EVENT_TYPE_DAMAGE, 0, 0, 1, 1;
	end 
	
	-- if ( powerType == POWERTYPE_STAMINA or powerType == POWERTYPE_FINESSE or powerType == POWERTYPE_MAGICKA ) then
		-- return true;
	-- end

	-- blacklist certain abilities ("Sprint Drain")
	-- if ( abilityName == "Sprint Drain" ) then
		-- return true;
	-- end
	
	return EVENT_TYPE_IGNORE, 0, 0, 0, 0;
end

--[[

DamageType
-----
DAMAGE_TYPE_COLD
DAMAGE_TYPE_DISEASE
DAMAGE_TYPE_DROWN
DAMAGE_TYPE_EARTH
DAMAGE_TYPE_FIRE
DAMAGE_TYPE_GENERIC
DAMAGE_TYPE_MAGIC
DAMAGE_TYPE_NONE
DAMAGE_TYPE_OBLIVION
DAMAGE_TYPE_PHYSICAL
DAMAGE_TYPE_POISON
DAMAGE_TYPE_SHOCK


CombatMechanicType
-----
POWERTYPE_FINESSE
POWERTYPE_HEALTH
POWERTYPE_INVALID
POWERTYPE_MAGICKA
POWERTYPE_MOUNT_STAMINA
POWERTYPE_STAMINA
POWERTYPE_ULTIMATE
POWERTYPE_WEREWOLF


CombatUnitType
-----
COMBAT_UNIT_TYPE_GROUP
COMBAT_UNIT_TYPE_NONE
COMBAT_UNIT_TYPE_OTHER
COMBAT_UNIT_TYPE_PLAYER
COMBAT_UNIT_TYPE_PLAYER_PET

]]--

function Recount.DebugStringifyUnitType( unitType )
	if ( unitType == COMBAT_UNIT_TYPE_PLAYER ) then
		return "CUT_PLAYER";
	elseif ( unitType == COMBAT_UNIT_TYPE_PLAYER_PET ) then
		return "CUT_PLAYER_PET";
	elseif ( unitType == COMBAT_UNIT_TYPE_GROUP ) then
		return "CUT_GROUP";
	elseif ( unitType == COMBAT_UNIT_TYPE_OTHER ) then
		return "CUT_OTHER";
	elseif ( unitType == COMBAT_UNIT_TYPE_NONE ) then
		return "CUT_NONE";
	else
		return "CUT_UNKNOWN";
	end
end


--[[==========================================
	CombatEvent

	Message handler
	==========================================]]--
function Recount.CombatEvent( eventCode, result, isError, abilityName, abilityGraphic, abilityActionSlotType, sourceName, sourceType, targetName, targetType, hitValue, powerType, damageType, log )
	-- check if we collect data in background
	if ( Recount.settings.suspendDataCollectionWhileHidden and Recount.settings.wndMain.isVisible == false ) then
		return;
	end

	-- update Recount time pointer
	Recount.time = GetGameTimeMilliseconds();
	
	if ( Recount.isDebugMode ) then
		debugString = sourceName .." [".. Recount.DebugStringifyUnitType( sourceType ) .."] <==> ".. targetName.. " [".. Recount.DebugStringifyUnitType( targetType ) .."] == (".. abilityName .."; ".. hitValue ..")<== ".. "; ".. abilityActionSlotType .."; ".. powerType .."; ".. damageType .. "; ".. result;
		RecountUIDebug_lblCombatLog:AddMessage( debugString );
	end
			
	local  eventType, hitCount, critCount, dotCount, dotCritCount = Recount.CheckIgnoreCombatEvent( result, isError, hitValue, powerType, damageType );
	if ( eventType == EVENT_TYPE_IGNORE ) then
		return;
	end
	
	
	
	--if player do or take damage, and is not in combat, then create a combat session
	local playerInCombat = (sourceType == COMBAT_UNIT_TYPE_PLAYER or targetType == COMBAT_UNIT_TYPE_PLAYER)	
	local selfEvent = sourceType == targetType
	if (Recount.inCombat == false) and (playerInCombat == true) and	(eventType == EVENT_TYPE_DAMAGE) and (sourceType ~= targetType) then
		Recount.playerEnterCombat()
	end
	
	
	
	--are we in a real combat segment?
	if Recount.inCombat == true then
		-- check if we should care about this combat event
		if ( sourceType ~= COMBAT_UNIT_TYPE_PLAYER and targetType ~= COMBAT_UNIT_TYPE_PLAYER and
			 sourceType ~= COMBAT_UNIT_TYPE_PLAYER_PET and targetType ~= COMBAT_UNIT_TYPE_PLAYER_PET and
			 sourceType ~= COMBAT_UNIT_TYPE_GROUP and targetType ~= COMBAT_UNIT_TYPE_GROUP ) then
			return;
		end

		

		playerUnit.total.timeSpan = Recount.time - playerUnit.total.timeStart;
		playerUnit.session.timeSpan = Recount.time - playerUnit.session.timeStart;
						
		-- Player is the TARGET
		if ( targetType == COMBAT_UNIT_TYPE_PLAYER ) then	
			if ( eventType == EVENT_TYPE_DAMAGE ) then	
				UnitDamageTaken( 1, abilityName, sourceName, targetName, hitValue, result);
			elseif ( eventType == EVENT_TYPE_HEAL ) then
				UnitHealingTaken( 1, abilityName, sourceName, targetName, hitValue, result);
			end
		end
		
		-- Player/Pet is the SOURCE	
		if ( sourceType == COMBAT_UNIT_TYPE_PLAYER or sourceType == COMBAT_UNIT_TYPE_PLAYER_PET ) then	
			
			if ( eventType == EVENT_TYPE_HEAL ) then
				UnitHealingDone( 1, abilityName, targetName, hitValue , result, hitCount, critCount, dotCount, dotCritCount );		
			elseif ( eventType == EVENT_TYPE_DAMAGE ) then
				UnitDamageDone( 1, abilityName, sourceName, targetName, hitValue , result, hitCount, critCount, dotCount, dotCritCount );			
			end
		end			
	end

	Recount.UpdateUI(Recount.refreshThreshold);
end

function AddAbilityValue( abilityList, abilityName, Value, c, cc, dc, dcc  )	
	if ( abilityList[abilityName] == nil ) then
		abilityList[abilityName] = {hitValue = Value, hitCount = c, critCount = cc, dotCount = dc, dotCritCount = dcc};
	else
		local skill = abilityList[abilityName];
		skill.hitValue = skill.hitValue + Value
		skill.hitCount = skill.hitCount + c
		skill.critCount = skill.critCount +  cc
		skill.dotCount = skill.dotCount + dc
		skill.dotCritCount = skill.dotCritCount + dcc
	end

end

local function getTag(b, tag)
	if b then
		return tag
	else
		return ""
	end
end

function nameCleanup(sName)
	if sName== nil or sName=="" then
		return "Unknown"		
	else
		return sName:match("([^\^]+)")
	end
end

--[[==========================================
	UnitDamageDone
	==========================================]]--
function UnitDamageDone( unitID, abilityName, sourceName, targetName, hitValue, result, hitCount, critCount, dotCount, dotCritCount )	
	local u = Recount.trackedUnits[unitID];
	
	if sourceName ~= targetName	then
		u.session.damageDone = u.session.damageDone + hitValue;	
		u.session.damageCount = u.session.damageCount + hitCount + dotCount;
		u.session.critDamageCount = u.session.critDamageCount + critCount + dotCritCount;
		u.total.damageDone = u.total.damageDone + hitValue;
		u.total.damageCount = u.total.damageCount + hitCount + dotCount;
		u.total.critDamageCount = u.total.critDamageCount + critCount + dotCritCount;	
	end
	
	AddAbilityValue( u.total.damagePerAbility, abilityName, hitValue, hitCount, critCount, dotCount, dotCritCount );
	AddAbilityValue( u.session.damagePerAbility, abilityName, hitValue, hitCount, critCount, dotCount, dotCritCount );
	
	
	local eventString = string.format( "|ccccc55 %s%d%s|r -> |c883333 %s|r - %s", getTag(dotCount == 1, Recount.settings.dotTag), hitValue, getTag((critCount == 1 ) or (dotCritCount == 1), Recount.settings.critTag) , nameCleanup(targetName), nameCleanup(abilityName) );
	Recount.CombatLogMsg( eventString );
end



--[[==========================================
	UnitDamageTaken
	==========================================]]--
function UnitDamageTaken( unitID, abilityName, sourceName, targetname, hitValue, result )
	
	local u = Recount.trackedUnits[unitID];

	u.total.damageTaken = u.total.damageTaken + hitValue;
	u.session.damageTaken = u.session.damageTaken + hitValue;	
	
	if sourceName ~= targetName then
		local eventString = string.format( "|ccc4444 %d|r <- |c883333 %s|r - %s", hitValue, nameCleanup(sourceName), nameCleanup(abilityName));
		Recount.CombatLogMsg( eventString );
	end
end

--[[==========================================
	UnitHealingDone
	==========================================]]--
function UnitHealingDone( unitID, abilityName, targetName, hitValue, result, hitCount, critCount, dotCount, dotCritCount  )
	
	local u = Recount.trackedUnits[unitID];

	u.session.healingDone = u.session.healingDone + hitValue;	
	u.session.healingCount = u.session.healingCount + hitCount + dotCount;
	u.session.critHealingCount = u.session.critHealingCount + critCount + dotCritCount;	
	u.total.healingDone = u.total.healingDone + hitValue;
	u.total.healingCount = u.total.healingCount +  hitCount + dotCount;
	u.total.critHealingCount = u.total.critHealingCount + critCount + dotCritCount;

	AddAbilityValue( u.total.healPerAbility, abilityName, hitValue, hitCount, critCount, dotCount, dotCritCount );
	AddAbilityValue( u.session.healPerAbility, abilityName, hitValue, hitCount, critCount, dotCount, dotCritCount );


	local eventString = string.format( "|c449944 %s%d%s|r -> |c447744 %s|r - %s", getTag(dotCount >0, Recount.settings.dotTag) , hitValue, getTag((critCount > 0) or (dotCritCount > 0), Recount.settings.critTag)  ,nameCleanup(targetName), nameCleanup(abilityName) );	
	Recount.CombatLogMsg( eventString );

end        

--[[==========================================
	UnitHealingTaken
	==========================================]]--
function UnitHealingTaken( unitID, abilityName, sourceName, targetName, hitValue, result )
	
	local u = Recount.trackedUnits[unitID];

	u.total.healingTaken = u.total.healingTaken + hitValue;
	u.session.healingTaken = u.session.healingTaken + hitValue;

	if sourceName ~= targetName then
		local eventString = string.format( "|c449944 %d|r <- |c447744 %s|r - %s", hitValue, nameCleanup(targetName), nameCleanup(abilityName));
		Recount.CombatLogMsg( eventString );
	end
end


function Recount.StartNewSession()

	-- currently only use one session
	-- playerUnit.sessionCurrent = playerUnit.sessionCurrent + 1;
	
	if ( playerUnit.sessionCurrent > Recount.settings.numStoredSessions ) then
		playerUnit.sessionCurrent = 1;		
	end
	
	playerUnit.session ={}

	playerUnit.sessionList[ playerUnit.sessionCurrent ] = {
		damageDone = 0;
		damageTaken = 0;
		healingDone = 0;
		healingTaken = 0;
		
		--crit
		healingCount = 0;
		critHealingCount = 0;
		damageCount = 0;
		critDamageCount = 0;
		
		dps = 0;
		hps = 0;

		timeStart = Recount.time;
		timeSpan = 1;

		damagePerAbility = {};
		healPerAbility = {};
	}

	playerUnit.session = playerUnit.sessionList[ playerUnit.sessionCurrent ];

	Recount.ClearProgressBars();

end

function Recount.Reset()

	playerUnit.total.timeStart = 0;	
	playerUnit.total.damageDone = 0;
	playerUnit.total.damageTaken = 0;
	playerUnit.total.healingTaken = 0;
	playerUnit.total.healingDone = 0;
	playerUnit.total.damagePerAbility = {};
	playerUnit.total.healPerAbility = {};
	--crit
	playerUnit.total.healingCount = 0;
	playerUnit.total.critHealingCount = 0;
	playerUnit.total.damageCount = 0;
	playerUnit.total.critDamageCount = 0;
	
	playerUnit.sessionList = {};	
	playerUnit.sessionCurrent = 1;
	playerUnit.sessionNum = 0;
	Recount.StartNewSession();
	Recount.ClearProgressBars();

	Recount.CombatLogMsg( "==== RESET ====" );

end

function Recount.ClearProgressBars()
	for i = 1, UI_STATUSBAR_MAX do
		_G["Statusbar"..i]:SetValue( 0 );
		_G["Statusbar"..i.."_lblLeft"]:SetText( "" );
	end

end


function Recount.FillProgressBars( abilityTable, totalValue )
	if ( abilityTable == nil or totalValue == nil ) then
		return;
	end		

	local i = 1;
	local dMax = 0;	
	Recount.maxComputedBars = 1
	
	local keys = {}
	for k,v in pairs(abilityTable) do
	  table.insert(keys,{key=k,data=v})
	end

	table.sort(keys,function(a,b) return a.data.hitValue > b.data.hitValue end)
	
	for i,skill in ipairs(keys) do	

		if dMax == 0 then
			dMax = skill.data.hitValue
		end;

		local abilityName = skill.key:match("([^\^]+)"); --get rid of the string suffixe in some languages
		local dmgAbs = skill.data.hitValue;
		
		local fbString = ""	
		if Recount.settings.showSkillDetails == true then
			if skill.data.hitCount > 0 then
				fbString = ("%s |cFF7777 #hits: %d|r |cFF5555 crit: %.1f%%|r"):format (fbString, skill.data.hitCount, secureDivide(skill.data.critCount, skill.data.hitCount) * 100)
			end
			if  skill.data.dotCount > 0 then
				fbString = ("%s |cFF7777 #ticks: %d|r |cFF5555 crit: %.1f%%|r"):format (fbString, skill.data.dotCount, secureDivide(skill.data.dotCritCount, skill.data.dotCount) * 100)
			end
		end
		
		local barSize = secureDivide(dmgAbs , dMax) * 100 --size of the bar vs best skill
		local dmgRel = secureDivide(dmgAbs, totalValue) * 100 --%damage vs other skills
								
		if ( i <= UI_STATUSBAR_MAX ) then
			_G["Statusbar"..i]:SetValue( barSize );
			_G["Statusbar"..i.."_lblLeft"]:SetText( ("|caaaaff %.2d|r %s |ceeeeee %s|r |c999999 (%d %%) %s"):format(i, abilityName, Recount.FormatNumber( dmgAbs ), dmgRel, fbString ) );
			_G["Statusbar"..i]:SetColor( 0.8 - i*0.05, 0.8 - i*0.05, 0.3, 0.8 );
			Recount.maxComputedBars = Recount.maxComputedBars + 1
		end

		i = i + 1;
	end
	keys = {}
	Recount.FitStatusBars();

end

function Recount.FillSingleProgressBar( name, value )
	Statusbar1:SetValue( value );
	Statusbar1_lblLeft:SetText( ("%s: |ceeeeee %d|r"):format( name, value ) );
end

function Recount.UpdateModeUI()

	if ( Recount.settings.currentMode == RC_MODE_DAMAGE_DONE_TOTAL ) then
		Recount.FillProgressBars( playerUnit.total.damagePerAbility, playerUnit.total.damageDone );

	elseif ( Recount.settings.currentMode == RC_MODE_DAMAGE_DONE) then
		if ( playerUnit.session ~= nil ) then
			Recount.FillProgressBars( playerUnit.session.damagePerAbility, playerUnit.session.damageDone );
		end
		
	elseif ( Recount.settings.currentMode == RC_MODE_DAMAGE_TAKEN) then
		if ( playerUnit.session ~= nil ) then
			Recount.FillSingleProgressBar( "Damage Taken", playerUnit.session.damageTaken );
		end
		
	elseif ( Recount.settings.currentMode == RC_MODE_DAMAGE_TAKEN_TOTAL) then
		Recount.FillSingleProgressBar( "Damage Taken", playerUnit.total.damageTaken );

	elseif ( Recount.settings.currentMode == RC_MODE_HEALING_DONE) then
		if ( playerUnit.session ~= nil ) then
			Recount.FillProgressBars( playerUnit.session.healPerAbility, playerUnit.session.healingDone );
		end		

	elseif ( Recount.settings.currentMode == RC_MODE_HEALING_DONE_TOTAL) then
		Recount.FillProgressBars( playerUnit.total.healPerAbility, playerUnit.total.healingDone );
		
	elseif ( Recount.settings.currentMode == RC_MODE_HEALING_TAKEN) then
		if ( playerUnit.session ~= nil ) then
			Recount.FillSingleProgressBar( "Healing Taken", playerUnit.session.healingTaken );
		end
		
	elseif ( Recount.settings.currentMode == RC_MODE_HEALING_TAKEN_TOTAL) then
		Recount.FillSingleProgressBar( "Healing Taken", playerUnit.total.healingTaken );
	end

end

function Recount.SetMode( mode, doUpdate)
	if (mode == -1) then
		mode =  RC_MODE_COMBAT_LOG
	end
	
	if ( mode == RC_MODE_EOL ) then
		mode = RC_MODE_DAMAGE_DONE;
	end
	Recount.settings.currentMode = mode;
	Recount.ClearProgressBars();

	RecountUI_Title_Text:SetText( Recount.name .. " - " .. modeStrings[mode] );

	RecountUI_Title:SetHidden( false );
	RecountUI_lblCombatLogBG:SetHidden( false );

	if ( mode == RC_MODE_COMBAT_LOG ) then
		for i = 1, UI_STATUSBAR_MAX do
			_G["Statusbar"..i]:SetHidden( true );
		end
		RecountUI_lblCombatLog:SetHidden( false );	
		return;
	end	

	if ( mode == RC_MODE_MINIMIZED ) then
		for i = 1, UI_STATUSBAR_MAX do
			_G["Statusbar"..i]:SetHidden( true );
		end
		RecountUI_lblCombatLog:SetHidden( true );	
		RecountUI_Title:SetHidden( true );
		RecountUI_lblCombatLogBG:SetHidden( true );
		return;		
	end	
	
	Recount.totalMode = (mode == RC_MODE_DAMAGE_DONE_TOTAL)  or (mode == RC_MODE_DAMAGE_TAKEN_TOTAL) or (mode == RC_MODE_HEALING_DONE_TOTAL) or (mode == RC_MODE_HEALING_TAKEN_TOTAL);

	for i = 1, UI_STATUSBAR_MAX do
		_G["Statusbar"..i]:SetHidden( false );
	end
	RecountUI_lblCombatLog:SetHidden( true );
	if doUpdate then
		Recount.UpdateUI(0);
	end

end

function Recount.TestProgressBars()
	Recount.SetMode( RC_MODE_DAMAGE_DONE, true );
	local test = {}
	for i = 1, 25 do
		test["Ability"..i] = {hitValue = math.random(500000), hitCount = math.random(50), critCount = math.random(50), dotCount = math.random(5), dotCritCount = math.random(5)}
	end	
	Recount.FillProgressBars( test, 1460000 );
end

function Recount.ToggleNextMode(value)
	if not RecountUI:IsHidden() then
		Recount.SetMode( Recount.settings.currentMode + value, true );
	end
end

function Recount.OnClickedTitleButton( self, button )
	Recount.ToggleNextMode(1)
end

function Recount.OnClickedTitleButtonReset( self, button )
	Recount.Reset();
end

function Recount.OnClickedTitleButtonHide( self, button )
	Recount.Hide();
end

function Recount.autoToggle(oldState, newState)	
		if(newState == SCENE_SHOWN) then
			Recount.UIinMenu = false
			if (Recount.settings.wndMain.isVisible == true) then
				RecountUI:SetHidden( false );
			end
		elseif(newState == SCENE_HIDING) then
			Recount.UIinMenu = true
			RecountUI:SetHidden( true );
		end
end

function Recount.Show()
	RecountUI:SetHidden( false );
	Recount.settings.wndMain.isVisible = true;
end

function Recount.Hide()
	EVENT_MANAGER:UnregisterForUpdate("hideOOC")
	RecountUI:SetHidden( true );
	Recount.settings.wndMain.isVisible = false;
end

function Recount.toggleShow()
	if Recount.UIinMenu == false then
		if Recount.settings.wndMain.isVisible == true then
			Recount.Hide()
		else
			Recount.Show();
		end
	end
end

function Recount.SlashCommands( text )

	if ( text == "" ) then
		d( "Recount " .. Recount.versionString );
		d( "Command line options: hide; show; min; restore; reset;" );
		return
	end

	if ( text == "hide" ) then
		Recount.Hide();
		return
	end

	if ( text == "show" ) then
		Recount.Show();
		return
	end

	if ( text == "min" ) then
		Recount.MinimizeWindow()
		return
	end

	if ( text == "restore" ) then
		Recount.RestoreWindow(true)
		return
	end

	if ( text == "mode" ) then
		Recount.ToggleNextMode()
		return
	end

	if ( text == "reset" ) then
		Recount.Reset();
		return
	end

	if ( text == "debug" ) then
		if ( Recount.isDebugMode ) then
			RecountUIDebug:SetHidden( true );
			Recount.isDebugMode = false;
		else
			RecountUIDebug:SetHidden( false );
			Recount.isDebugMode = true;
		end
		return
	end
		
	if ( text == "test" ) then
		Recount.TestProgressBars();
		return
	end
end

ZO_CreateStringId("SI_BINDING_NAME_RECOUNT_SHOW", "Toggle UI on/off") 
ZO_CreateStringId("SI_BINDING_NAME_RECOUNT_RESET", "Reset Stats") 
ZO_CreateStringId("SI_BINDING_NAME_RECOUNT_TOGGLEMODE", "Toggle Mode (Forward)") 
ZO_CreateStringId("SI_BINDING_NAME_RECOUNT_TOGGLEMODE2", "Toggle Mode (Backward") 
