
Recount.SettingsMenu = {
	name = "Recount_SettingsMenu",
}

LAM = LibStub("LibAddonMenu-1.0")		
function Recount.SettingsMenu.Initialize()

	Recount.SettingsMenu.id = LAM:CreateControlPanel( Recount.SettingsMenu.name, Recount.name .. " Settings" );
	Recount.SettingsMenu:Setup()

end


function Recount.SettingsMenu:Toggle( setting )
	Recount.settings[setting] = not Recount.settings[setting];
end

function Recount.SettingsMenu:Set( setting, value )
	Recount.settings[setting] = value;

	if ( setting == 'progressBarHeight' or setting == 'progressBarFont' ) then		
		Recount.FitUIElements();
	end
end

function Recount.SettingsMenu:SetBarColor( r, g, b, a )
	Recount.settings.progressBarColor.r = r;
	Recount.settings.progressBarColor.g = g;
	Recount.settings.progressBarColor.b = b;
	Recount.settings.progressBarColor.a = a;
	Recount.FitUIElements();
end

function NameID( name )
	return Recount.SettingsMenu.name .. "_" .. name;
end

fonts = {
"ZoFontAlert",
"ZoFontAnnounceSmall",
"ZoFontBookLetter",
"ZoFontBookPaper",
"ZoFontBookRubbing",
"ZoFontBookScroll",
"ZoFontBookSkin",
"ZoFontBookTablet",
"ZoFontBoss",
"ZoFontBossName",
"ZoFontCenterScreenAnnounceSmall",
"ZoFontChat",
"ZoFontConversationOption",
"ZoFontConversationText",
"ZoFontCreditsHeader",
"ZoFontDialogKeybindDescription",
"ZoFontEdit",
"ZoFontEditChat",
"ZoFontGame",
"ZoFontGameBold",
"ZoFontGameLarge",
"ZoFontGameLargeBold",
"ZoFontGameMedium",
"ZoFontGameOutline",
"ZoFontGameShadow",
"ZoFontGameSmall",
"ZoFontHeader",
"ZoFontHeader2",
"ZoFontKeybindStripKey",
"ZoFontTooltipSubtitle",
"ZoFontTooltipTitle",
"ZoFontWindowSubtitle",
"ZoFontWinH2",
"ZoFontWinH3",
"ZoFontWinH4",
"ZoFontWinH5",
"ZoFontWinT1",
"ZoFontWinT2",
}

function Recount.SettingsMenu:Setup()

	local header = LAM:AddHeader( Recount.SettingsMenu.id, NameID( "Subtitle" ), "Version "  .. Recount.versionString );
	
	LAM:AddCheckbox( 
						Recount.SettingsMenu.id,
						NameID( "OnlyShowWithKey" ),
						"Disable all auto show/hide",
						"Recount UI does not auto toggle itself, but only through command shortcuts",
						function() return Recount.settings.onlyUseKey end,
						function() Recount.SettingsMenu:Toggle( 'onlyUseKey' ) end,
						false, nil );

	LAM:AddCheckbox( Recount.SettingsMenu.id, NameID( "Minimize" ), "Minimize in Combat", "Minimize the main window while in combat?", function() return Recount.settings.minimizeDuringCombat end, function() Recount.SettingsMenu:Toggle( 'minimizeDuringCombat' ) end, false, nil );
	LAM:AddCheckbox( Recount.SettingsMenu.id, NameID( "HideOOC" ), "Hide while Out Of Combat", "Hide the main window while out of combat?", function() return Recount.settings.hideOutOfCombat end, function() Recount.SettingsMenu:Toggle( 'hideOutOfCombat' ) end, false, nil );
	LAM:AddSlider( Recount.SettingsMenu.id , NameID( "HideOOCDelay" ), "OOC Hide Delay", "Adjust after what time the main window will be hidden when out of combat.", 1, 30, 1, function() return Recount.settings.hideOutOfCombatDelayInSeconds end, function( value ) Recount.SettingsMenu:Set( 'hideOutOfCombatDelayInSeconds', value ) end );
	
	LAM:AddCheckbox( Recount.SettingsMenu.id, NameID( "Autoswitch" ), "Autoswitch Mode", "Autoswitch between log mode and damage overview after combat?", function() return Recount.settings.autoSwitchMode end, function() Recount.SettingsMenu:Toggle( 'autoSwitchMode' ) end, false, nil );
	LAM:AddCheckbox( Recount.SettingsMenu.id, NameID( "StopHidden" ), "Suspend Data Collection while Hidden", "Do not continue to collect data while the main window is hidden", function() return Recount.settings.suspendDataCollectionWhileHidden end, function() Recount.SettingsMenu:Toggle( 'suspendDataCollectionWhileHidden' ) end, false, nil );

	LAM:AddCheckbox( Recount.SettingsMenu.id, NameID( "CountDeflectedDmg" ), "Count deflected Damage", "Consider deflected (parried, immune, reflected, ...) damage as normal damage done?", function() return Recount.settings.countDeflectedDmg end, function() Recount.SettingsMenu:Toggle( 'countDeflectedDmg' ) end, false, nil );
	
	LAM:AddCheckbox( 
						Recount.SettingsMenu.id,
						NameID( "ShowSkillDetails" ),
						"Show skills details",
						"Show hit number and critical strike statistics in skill bar",
						function() return Recount.settings.showSkillDetails end,
						function() Recount.SettingsMenu:Toggle( 'showSkillDetails' ) end,
						false, nil );
						
	
	

	LAM:AddSlider( 
					Recount.SettingsMenu.id ,
					NameID( "ProgBarHeight" ),
					"Damage Bar Height",
					"Height of the damage bars.",
					10, 40, 1, 
					function() return Recount.settings.progressBarHeight end,
					function( value ) Recount.SettingsMenu:Set( 'progressBarHeight', value ) end 
				);

	LAM:AddDropdown( Recount.SettingsMenu.id , NameID( "BarFont" ), "Damage Bar Font", "Damage Bar Font.", fonts, function() return Recount.settings.progressBarFont end, function( value ) Recount.SettingsMenu:Set( 'progressBarFont', value ) end );

	LAM:AddColorPicker( Recount.SettingsMenu.id , NameID( "BarColor" ), "Damage Bar Color", "Damage Bar Color.", function() return Recount.settings.progressBarColor.r, Recount.settings.progressBarColor.g, Recount.settings.progressBarColor.b, Recount.settings.progressBarColor.a end, function( r, g, b, a ) Recount.SettingsMenu:SetBarColor( r, g, b, a ) end );
	
	
	LAM:AddEditBox(
					Recount.SettingsMenu.id,
					NameID( "setDotTag" ),
					"Periodic tick tag",
					"How periodic ticks are shown in the combat log",
					false,
					function() return Recount.settings.dotTag end,
					function(value) Recount.settings.dotTag = value end,					
					false, 
					nil
				)
				
	LAM:AddEditBox(
					Recount.SettingsMenu.id,
					NameID( "setCritTag" ),
					"Critical strike Tag",
					"How critical strikes are shown in the combat log",
					false,
					function() return Recount.settings.critTag end,
					function(value) Recount.settings.critTag = value end,
					false, 
					nil
				)

end